

#include  <stdio.h>
#include  <stdlib.h>
#include  <unistd.h>
#include  <math.h>
#include  <sndfile.h>

#include  <tthimage.h>

#include "globals.h"

#include  "funcs.h"
#include  "parseproj.h"

/* ---------------------------------------------------------- */

#define X 105
#define Y  40
#define W 640
#define H 512

#define DO_RAW_DATA	0
#define BASE_FREQ	1111

int	verbosity;		/* really global var */

#if DO_RAW_DATA
static FILE		*dfile;
#endif

A_project		project;

static float		*sndfbuff = NULL;


/* ---------------------------------------------------------- */
int gen_sound_frame(SNDFILE *sf, float flv, float frv, float lvl)
{
// #define 		NB_FR (SAMPLE_RATE/FPS)
int			nbr_frames = SAMPLE_RATE/project.fps;
int			idx;
static long		lidx = 0L; 
float			fidx, fk;
float			d_lv, d_rv,		/* deltas */
			inc_l, inc_r,		/* increments */
			to_l, to_r;		/* target */
			
static float		prev_lv = 0.0;
static float		prev_rv = 0.0;

float			freq, fval;

if (verbosity)
	fprintf(stderr, "lv,lr  %6.3f  %6.3f   prev %6.3f %6.3f\n",
		flv, frv,	prev_lv, prev_rv);

d_lv = flv - prev_lv;	inc_l = d_lv / (float)nbr_frames;
d_rv = frv - prev_rv;	inc_r = d_rv / (float)nbr_frames;

if (verbosity) fprintf(stderr, "deltas %f %f incs %g %g\n",
				d_lv, d_rv, inc_l, inc_r);
to_l = prev_lv;	to_r = prev_rv;

fk = 0.00006;

freq = BASE_FREQ*fk;

for (idx=0; idx<nbr_frames; idx++) {

/* XXX	fidx = (BASE_FREQ+(to_l*4.20))*idx*fk;
	buffer[2*idx]   = lvl * sin(fidx);
	to_l += inc_l; */

	fval = (puissance256(to_l)/256.0);
	sndfbuff[2*idx] = fval * sin(freq*lidx);
	to_l += inc_l;
	
	fval = (puissance256(to_r)/256.0);
	sndfbuff[2*idx+1] = fval * sin(freq*lidx*1.09);
	to_r += inc_r;
#if DO_RAW_DATA
	fprintf(dfile, "%f %f\n", frv/256.0, fval);
#endif

	lidx++;
	}

sf_write_float (sf, sndfbuff, 2 * nbr_frames);
fprintf(stderr, "Got:\t%10.6f  %10.6f\n", to_l, to_r);

prev_lv = flv, prev_rv = frv;

return 0;
}
/* ---------------------------------------------------------- */
int grande_boucle(int notused)
{
int		idx, foo;
char		fname[100];
Image_Desc	*img, *out;
int		srcwidth, srcheight;
float		values[6], level;
Image_Rect	pict, rect;
SNDFILE		*wav;
float		leftv, rightv;
double		debut, fin;		/* mesure du temps */

fprintf(stderr, ">>>>>> grande boucle\n");

pict.x = X;		pict.y = Y;
pict.w = W;		pict.h = H;

debut = dtime();

/* precompute some invarients */
foo = SAMPLE_RATE / project.fps;
#if DEBUG_LEVEL
fprintf(stderr, "samples per frames : %d\n", foo);
#endif

if (NULL==(sndfbuff =malloc(foo*sizeof(float)*2))) {
	fprintf(stderr, "%s : no mem for sndbuff\n", __func__);
	abort();
	}

if (NULL==(wav=openwav("foo.wav"))) {
	fprintf(stderr, "ERR %s : out wav file\n", __func__);
	exit(1);
	}

#if DO_RAW_DATA
if (NULL==(dfile=fopen("raw.datas", "wb"))) {
	perror("raw data file");
	exit(1);
	}
#endif

sprintf(fname, "T/92/%05d.tga", 0);
Image_TGA_get_dims(fname, &srcwidth, &srcheight);
fprintf(stderr, "taille source : %d x %d\n", srcwidth, srcheight);

level = 0.92;

for (idx=0; idx<10000; idx++) {
	sprintf(fname, "T/92/%05d.tga", idx);
		
	img=Image_TGA_alloc_load(fname);
	if (NULL==img) {
		fprintf(stderr, "NO MORE FILE\n");
		break;
		}

	if (verbosity) fprintf(stderr, "%s -> %p\n", fname, img);

	if (NULL==(out=Image_create_subimg(img, &pict, 0))) {
		fprintf(stderr, "ABEND\n");
		abort();
		}

	Image_DeAllocate(img);		free(img);

	rect.x = 162;		rect.y = 44;
	rect.w = 274;		rect.h = 68;

	foo = mesure_zone(out, &rect, values);
	if (foo) {
		fprintf(stderr, "ERR mesure zone %d\n", foo);
		abort();
		}
	leftv = values[2];
	trace_zone(out, &rect, leftv);

	rect.x = 202;		rect.y = 207;
	rect.w = 106;		rect.h = 204;

	foo = mesure_zone(out, &rect, &values[3]);
	if (foo) {
		fprintf(stderr, "ERR mesure zone %d\n", foo);
		abort();
		}
	rightv = values[5];
	trace_zone(out, &rect, rightv);

	foo = gen_sound_frame(wav, leftv, rightv, level);
	if (foo) {
		exit(1);
		}

#if DO_RAW_DATA
	printf("%4d %f %f %f %f %f %f\n", idx,
			values[0], values[1], values[2],
			values[3], values[4], values[5]);
#endif

	sprintf(fname, "sp/%05d.tga", idx);
	if (verbosity) {
		Image_trace_chaine_1(out, fname, 10, 10,
			"8x8thin", &project.paper, &project.ink);
		}
	foo = Image_TGA_save(fname, out, 0);
	Image_DeAllocate(out);		free(out);

	if (verbosity)	fputs("\n", stderr);
	}

closewav(wav);
#if DO_RAW_DATA
fclose(dfile);
#endif

fin = dtime();
fprintf(stderr, "*\n* idx %5d  t %8.3f s   =  %6.1f fps\n*\n",
		idx, fin-debut, (double)idx/(fin-debut));

return 0;
}
/* ---------------------------------------------------------- */

char *strdup(const char*);		/* HORRIBLE HACK */

int main(int argc, char *argv[])
{
int		opt;
char		options[] = "vP:";
int		foo;
char		*projname = "essai.projet";

while (-1!=(opt = getopt(argc, argv, options))) {
	switch (opt) {
		case 'v':	verbosity++;		break;
		case 'P':	projname = strdup(optarg);
				break;
		
		default:				exit(1);
		}
	}

foo = parse_project(projname, &project);
if (foo)	exit(1);
print_project(&project, projname, 0);

(void)grande_boucle(0);

return 0;
}
/* ---------------------------------------------------------- */
